@@ -58,7 +58,7 @@ import org.apache.cxf.message.MessageUtils;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.http.Address;
 import org.apache.cxf.transport.http.Headers;
-import org.apache.cxf.transport.http.URLConnectionHTTPConduit;
+import org.apache.cxf.transport.http.HttpClientHTTPConduit;
 import org.apache.cxf.transport.https.HttpsURLConnectionInfo;
 import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
 import org.apache.cxf.version.Version;
@@ -79,7 +79,7 @@ import io.netty.handler.codec.http.HttpResponse;
 import io.netty.handler.ssl.SslHandler;
 
 
-public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLifeCycleListener {
+public class NettyHttpConduit extends HttpClientHTTPConduit implements BusLifeCycleListener {
     /**
      * Enable HTTP/2 support
      */
@@ -591,7 +591,10 @@ public class NettyHttpConduit extends URLConnectionHTTPConduit implements BusLif
 
         @Override
         protected void handleResponseAsync() throws IOException {
-            isAsync = true;
+            // The response hasn't been handled yet, should be handled asynchronously
+            if (httpResponse == null) {
+                isAsync = true;
+            }
         }
 
         @Override
